// Импорт класса ItemStack из модуля Minecraft для работы с предметами.
import { ItemStack } from "@minecraft/server";

// Экспортируемый объект badEffects содержит список негативных эффектов, которые могут быть применены к игроку.
// Каждый эффект включает название эффекта, длительность (в секундах) и уровень усиления.
export const badEffects = {
   0: { "effect": "weakness", "duration": 9, "amplifier": 0 }, // Слабость, длительность 9 секунд, уровень 0.
   1: { "effect": "weakness", "duration": 4, "amplifier": 1 }, // Слабость, длительность 4 секунды, уровень 1.
   2: { "effect": "slowness", "duration": 5, "amplifier": 0 }, // Замедление, длительность 5 секунд, уровень 0.
   3: { "effect": "slowness", "duration": 3, "amplifier": 1 }, // Замедление, длительность 3 секунды, уровень 1.
   4: { "effect": "slowness", "duration": 2, "amplifier": 2 }, // Замедление, длительность 2 секунды, уровень 2.
   5: { "effect": "blindness", "duration": 7, "amplifier": 0 }, // Слепота, длительность 7 секунд, уровень 0.
   6: { "effect": "darkness", "duration": 7, "amplifier": 0 }  // Тьма, длительность 7 секунд, уровень 0.
};

// Функция для замены предмета в руке игрока на новый с сохранением свойств.
// @param playerData - Объект игрока.
// @param heldItem - Текущий предмет в руке.
// @param targetItem - Тип предмета, на который нужно заменить.
export function itemReplace(playerData, heldItem, targetItem) {
   // Проверяем, совпадает ли текущий предмет с целевым. Если да, выходим из функции.
   if (heldItem.typeId == targetItem) return;

   // Получаем инвентарь игрока.
   const inventory = playerData.getComponent('inventory').container;
   // Получаем список зачарований текущего предмета.
   const enchantArray = heldItem.getComponent("enchantable")?.getEnchantments();
   // Создаём новый предмет с указанным типом.
   const itemReplacement = new ItemStack(targetItem);
   // Получаем все динамические свойства текущего предмета.
   const dynamicProperties = heldItem.getDynamicPropertyIds();

   // Если есть зачарования, добавляем их к новому предмету.
   if (enchantArray) itemReplacement.getComponent("enchantable").addEnchantments(enchantArray);
   // Копируем текущий уровень повреждения предмета.
   itemReplacement.getComponent('durability').damage = heldItem.getComponent("durability").damage;
   // Копируем имя предмета.
   itemReplacement.nameTag = heldItem.nameTag;
   // Копируем описание (lore) предмета.
   itemReplacement.setLore(heldItem.getLore());
   // Копируем все динамические свойства.
   for (let i = 0; i < dynamicProperties.length; i++) {
      const currentPropertyValue = heldItem.getDynamicProperty(dynamicProperties[i]);
      itemReplacement.setDynamicProperty(dynamicProperties[i], currentPropertyValue);
   }
   // Устанавливаем новый предмет в текущий слот игрока.
   inventory.setItem(playerData.selectedSlotIndex, itemReplacement);
}

// Функция для запуска снаряда из арбалета.
// @param world - Объект мира.
// @param playerData - Объект игрока.
// @param projectile - Тип снаряда.
// @param power - Сила выстрела.
// @param multishotBonus - Бонус для мультивыстрела.
// @param isPiercing - Уровень пробивания.
// @param potionEffect - Эффект зелья, применяемый к снаряду.
// @param damage - Урон, наносимый снарядом.
export function shootProjectile(world, playerData, projectile, power, multishotBonus, isPiercing, potionEffect, damage) {
   // Создаём снаряд в мире на позиции игрока с небольшим смещением в направлении взгляда.
   let projectileToShoot = playerData.dimension.spawnEntity(projectile, {
      x: playerData.location.x + (playerData.getViewDirection().x),
      y: playerData.location.y + 1.7 + (playerData.getViewDirection().y),
      z: playerData.location.z + (playerData.getViewDirection().z)
   });
   // Получаем компонент снаряда.
   let projectileComp = projectileToShoot.getComponent("minecraft:projectile");
   // Устанавливаем владельца снаряда (игрока).
   projectileComp.owner = playerData;

   // Устанавливаем динамические свойства снаряда.
   projectileToShoot.setDynamicProperty('sag:damage', damage); // Урон.
   projectileToShoot.setDynamicProperty('sag:piercing', 0); // Начальный уровень пробивания.
   projectileToShoot.setDynamicProperty('sag:arrow_counter', 0); // Счётчик попаданий.
   projectileToShoot.setDynamicProperty('sag:potion_effect', potionEffect[0]); // Тип эффекта зелья.
   projectileToShoot.setDynamicProperty('sag:potion_duration', potionEffect[1]); // Длительность эффекта.
   projectileToShoot.setDynamicProperty('sag:potion_amplifier', potionEffect[2]); // Уровень эффекта.

   // Получаем направление взгляда игрока.
   const viewX = playerData.getViewDirection().x;
   const viewZ = playerData.getViewDirection().z;

   // Если пробивание отсутствует, применяем мультивыстрел с отклонением траектории.
   if (!isPiercing) {
      if ((viewX > 0 && viewZ > 0) || (viewX < 0 && viewZ < 0)) {
         projectileComp.shoot({
            x: (playerData.getViewDirection().x - multishotBonus) * power,
            y: playerData.getViewDirection().y * power,
            z: (playerData.getViewDirection().z + multishotBonus) * power
         });
      } else {
         projectileComp.shoot({
            x: (playerData.getViewDirection().x + multishotBonus) * power,
            y: playerData.getViewDirection().y * power,
            z: (playerData.getViewDirection().z + multishotBonus) * power
         });
      }
   } else {
      // Если есть пробивание, активируем событие пробивания и устанавливаем свойства.
      projectileToShoot.triggerEvent('sag:piercing');
      projectileToShoot.setDynamicProperty('sag:piercing', isPiercing);
      projectileToShoot.setDynamicProperty('sag:arrow_counter', 1);
      projectileComp.shoot({
         x: (playerData.getViewDirection().x) * power,
         y: playerData.getViewDirection().y * power,
         z: (playerData.getViewDirection().z) * power
      });
   }
   // Возвращаем созданный снаряд.
   return projectileToShoot;
}

// Функция для нанесения урона предмету в руке игрока.
// @param player - Объект игрока.
// @param damage - Количество урона, наносимого предмету.
export function damageItem(player, damage) {
   // Генерируем случайное число для имитации "взмаха" предметом.
   const fakeSwing = Math.floor(Math.random() * 100);
   // Получаем компонент экипировки игрока.
   const equipment = player.getComponent('equippable');
   // Получаем предмет в руке.
   const item = equipment.getEquipment('Mainhand');
   // Получаем инвентарь игрока.
   const inventory = player.getComponent('inventory').container;
   // Получаем компонент прочности предмета.
   const durability = item.getComponent('durability');
   // Получаем зачарование "Нерушимость", если оно есть.
   const enchant = item?.getComponent("minecraft:enchantable");
   const level = enchant?.getEnchantment("unbreaking")?.level ?? 0;

   if (item && durability) {
      // Устанавливаем свойство для имитации взмаха.
      item.setDynamicProperty('sag:swing', fakeSwing);

      // Если прочность предмета исчерпана, удаляем предмет и проигрываем звук поломки.
      if ((durability.maxDurability - durability.damage - damage) <= 0) {
         inventory.setItem(player.selectedSlotIndex);
         player.playSound('random.break');
      }

      // Если нет зачарования "Нерушимость", наносим полный урон.
      if (item && level == 0) {
         durability.damage += damage;
         inventory.setItem(player.selectedSlotIndex, item);
      }

      // Если есть "Нерушимость", с некоторой вероятностью урон не наносится.
      if (item && level > 0) {
         const rng = Math.floor(Math.random() * 5);
         durability.damage += rng <= level ? 0 : 1;
         inventory.setItem(player.selectedSlotIndex, item);
      }
   }
}

// Экспортируемый объект ARROW_EFFECTS содержит эффекты, которые могут быть применены к стрелам.
// Каждый эффект включает название, длительность (в секундах) и уровень усиления.
export const ARROW_EFFECTS = {
   6: ["night_vision", 22, 0], // Ночное зрение, 22 секунды, уровень 0.
   7: ["night_vision", 60, 0], // Ночное зрение, 60 секунд, уровень 0.
   8: ["invisibility", 22, 0], // Невидимость, 22 секунды, уровень 0.
   9: ["invisibility", 60, 0], // Невидимость, 60 секунд, уровень 0.
   10: ["jump_boost", 22, 0], // Прыгучесть, 22 секунды, уровень 0.
   11: ["jump_boost", 60, 0], // Прыгучесть, 60 секунд, уровень 0.
   12: ["jump_boost", 11, 1], // Прыгучесть, 11 секунд, уровень 1.
   13: ["fire_resistance", 22, 0], // Огнестойкость, 22 секунды, уровень 0.
   14: ["fire_resistance", 60, 0], // Огнестойкость, 60 секунд, уровень 0.
   15: ["speed", 22, 0], // Скорость, 22 секунды, уровень 0.
   16: ["speed", 60, 0], // Скорость, 60 секунд, уровень 0.
   17: ["speed", 11, 1], // Скорость, 11 секунд, уровень 1.
   18: ["slowness", 22, 0], // Замедление, 22 секунды, уровень 0.
   19: ["slowness", 60, 0], // Замедление, 60 секунд, уровень 0.
   20: ["water_breathing", 22, 0], // Подводное дыхание, 22 секунды, уровень 0.
   21: ["water_breathing", 60, 0], // Подводное дыхание, 60 секунд, уровень 0.
   22: ["instant_health", 1, 0], // Мгновенное исцеление, 1 секунда, уровень 0.
   23: ["instant_health", 1, 1], // Мгновенное исцеление, 1 секунда, уровень 1.
   24: ["instant_damage", 1, 0], // Мгновенный урон, 1 секунда, уровень 0.
   25: ["instant_damage", 1, 1], // Мгновенный урон, 1 секунда, уровень 1.
   26: ["poison", 5, 0], // Яд, 5 секунд, уровень 0.
   27: ["poison", 15, 0], // Яд, 15 секунд, уровень 0.
   28: ["poison", 5, 1], // Яд, 5 секунд, уровень 1.
   29: ["regeneration", 5, 0], // Регенерация, 5 секунд, уровень 0.
   30: ["regeneration", 15, 0], // Регенерация, 15 секунд, уровень 0.
   31: ["regeneration", 5, 1], // Регенерация, 5 секунд, уровень 1.
   32: ["strength", 22, 0], // Сила, 22 секунды, уровень 0.
   33: ["strength", 60, 0], // Сила, 60 секунд, уровень 0.
   34: ["strength", 11, 1], // Сила, 11 секунд, уровень 1.
   35: ["weakness", 22, 0], // Слабость, 22 секунды, уровень 0.
   36: ["weakness", 60, 0], // Слабость, 60 секунд, уровень 0.
   37: ["wither", 22, 0], // Иссушение, 22 секунды, уровень 0.
   38: ["resistance", 2, 2], // Сопротивление, 2 секунды, уровень 2.
   39: ["resistance", 5, 2], // Сопротивление, 5 секунд, уровень 2.
   40: ["resistance", 2, 4], // Сопротивление, 2 секунды, уровень 4.
   41: ["slow_falling", 22, 0], // Медленное падение, 22 секунды, уровень 0.
   42: ["slow_falling", 60, 0], // Медленное падение, 60 секунд, уровень 0.
   43: ["slowness", 11, 1], // Замедление, 11 секунд, уровень 1.
   44: ["wind_charged", 22, 0], // Заряженный ветром, 22 секунды, уровень 0.
   45: ["weaving", 22, 0], // Ткачество, 22 секунды, уровень 0.
   46: ["oozing", 22, 0], // Слизь, 22 секунды, уровень 0.
   47: ["infested", 22, 0] // Заражение, 22 секунды, уровень 0.
};